@@ -26,6 +26,7 @@
 #include <sys/stat.h>
 #include <sys/wait.h>
 
+#include <errno.h>
 #include <limits.h>
 #include <fcntl.h>
 #include <libgen.h>
@@ -89,6 +90,20 @@ deb_parse_control(const char *filename)
 	return pkg;
 }
 
+static time_t
+parse_timestamp(const char *value)
+{
+	time_t timestamp;
+	char *end;
+
+	errno = 0;
+	timestamp = strtol(value, &end, 10);
+	if (value == end || *end || errno != 0)
+		ohshite(_("unable to parse timestamp '%.255s'"), value);
+
+	return timestamp;
+}
+
 /* Cleanup filename for use in crippled msdos systems. */
 static char *
 clean_msdos_filename(char *filename)
@@ -117,6 +132,8 @@ mksplit(const char *file_src, const char *prefix, off_t maxpartsize,
 	struct dpkg_error err;
 	int fd_src;
 	struct stat st;
+	time_t timestamp;
+	const char *timestamp_str;
 	const char *version;
 	char hash[MD5HASHLEN + 1];
 	int nparts, curpart;
@@ -141,7 +158,13 @@ mksplit(const char *file_src, const char *prefix, off_t maxpartsize,
 	lseek(fd_src, 0, SEEK_SET);
 
 	pkg  = deb_parse_control(file_src);
-	version = versiondescribe(&pkg->available.version, vdew_always);
+	version = versiondescribe(&pkg->available.version, vdew_nonambig);
+
+	timestamp_str = getenv("SOURCE_DATE_EPOCH");
+	if (timestamp_str)
+		timestamp = parse_timestamp(timestamp_str);
+	else
+		timestamp = time(NULL);
 
 	partsize = maxpartsize - HEADERALLOWANCE;
 	last_partsize = st.st_size % partsize;
@@ -165,7 +188,7 @@ mksplit(const char *file_src, const char *prefix, off_t maxpartsize,
 	}
 
 	for (curpart = 1; curpart <= nparts; curpart++) {
-		int fd_dst;
+		struct dpkg_ar *ar;
 
 		varbuf_reset(&file_dst);
 		/* Generate output filename. */
@@ -196,12 +219,11 @@ mksplit(const char *file_src, const char *prefix, off_t maxpartsize,
 		}
 
 		/* Split the data. */
-		fd_dst = creat(file_dst.buf, 0644);
-		if (fd_dst < 0)
-			ohshite(_("unable to open file '%s'"), file_dst.buf);
+		ar = dpkg_ar_create(file_dst.buf, 0644);
+		dpkg_ar_set_mtime(ar, timestamp);
 
 		/* Write the ar header. */
-		dpkg_ar_put_magic(file_dst.buf, fd_dst);
+		dpkg_ar_put_magic(ar);
 
 		/* Write the debian-split part. */
 		varbuf_printf(&partmagic,
@@ -209,17 +231,17 @@ mksplit(const char *file_src, const char *prefix, off_t maxpartsize,
 		              SPLITVERSION, pkg->set->name, version, hash,
 		              (intmax_t)st.st_size, (intmax_t)partsize,
 		              curpart, nparts, pkg->available.arch->name);
-		dpkg_ar_member_put_mem(file_dst.buf, fd_dst, PARTMAGIC,
+		dpkg_ar_member_put_mem(ar, PARTMAGIC,
 		                       partmagic.buf, partmagic.used);
 		varbuf_reset(&partmagic);
 
 		/* Write the data part. */
 		varbuf_printf(&partname, "data.%d", curpart);
-		dpkg_ar_member_put_file(file_dst.buf, fd_dst, partname.buf,
+		dpkg_ar_member_put_file(ar, partname.buf,
 		                        fd_src, cur_partsize);
 		varbuf_reset(&partname);
 
-		close(fd_dst);
+		dpkg_ar_close(ar);
 
 		printf("%d ", curpart);
 	}
