@@ -34,8 +34,8 @@ DPKG_BEGIN_DECLS
 
 int pkg_sorter_by_nonambig_name_arch(const void *a, const void *b);
 
-const char *pkg_summary(const struct pkginfo *pkg, const struct pkgbin *pkgbin,
-                        int *len_ret);
+const char *pkgbin_summary(const struct pkginfo *pkg,
+                           const struct pkgbin *pkgbin, int *len_ret);
 int pkg_abbrev_want(const struct pkginfo *pkg);
 int pkg_abbrev_status(const struct pkginfo *pkg);
 int pkg_abbrev_eflag(const struct pkginfo *pkg);
