@@ -34,7 +34,7 @@ use Exporter qw(import);
 use Dpkg ();
 use Dpkg::ErrorHandling;
 use Dpkg::Gettext;
-use Dpkg::BuildEnv;
+use Dpkg::Build::Env;
 use Dpkg::Control::HashCore;
 
 my $origins = "$Dpkg::CONFDIR/origins";
@@ -61,13 +61,16 @@ the relationship by listing the base distribution in the Parent field:
 
   Parent: Debian
 
-The file should be named according to the vendor name.
+The file should be named according to the vendor name. The usual convention
+is to name the vendor file using the vendor name in all lowercase, but some
+variation is permitted. Namely, spaces are mapped to dashes ('-'), and the
+file can have the same casing as the Vendor field, or it can be capitalized.
 
 =head1 FUNCTIONS
 
 =over 4
 
-=item $dir = Dpkg::Vendor::get_vendor_dir()
+=item $dir = get_vendor_dir()
 
 Returns the current dpkg origins directory name, where the vendor files
 are stored.
@@ -78,7 +81,7 @@ sub get_vendor_dir {
     return $origins;
 }
 
-=item $fields = Dpkg::Vendor::get_vendor_info($name)
+=item $fields = get_vendor_info($name)
 
 Returns a Dpkg::Control object with the information parsed from the
 corresponding vendor file in $Dpkg::CONFDIR/origins/. If $name is omitted,
@@ -96,12 +99,12 @@ sub get_vendor_info(;$) {
     my $file = get_vendor_file($vendor);
     return unless $file;
     my $fields = Dpkg::Control::HashCore->new();
-    $fields->load($file) or error(g_('%s is empty'), $file);
+    $fields->load($file, compression => 0) or error(g_('%s is empty'), $file);
     $VENDOR_CACHE{$vendor} = $fields;
     return $fields;
 }
 
-=item $name = Dpkg::Vendor::get_vendor_file($name)
+=item $name = get_vendor_file($name)
 
 Check if there's a file for the given vendor and returns its
 name.
@@ -121,7 +124,7 @@ sub get_vendor_file(;$) {
     return $file;
 }
 
-=item $name = Dpkg::Vendor::get_current_vendor()
+=item $name = get_current_vendor()
 
 Returns the name of the current vendor. If DEB_VENDOR is set, it uses
 that first, otherwise it falls back to parsing $Dpkg::CONFDIR/origins/default.
@@ -131,8 +134,8 @@ If that file doesn't exist, it returns undef.
 
 sub get_current_vendor() {
     my $f;
-    if (Dpkg::BuildEnv::has('DEB_VENDOR')) {
-        $f = get_vendor_info(Dpkg::BuildEnv::get('DEB_VENDOR'));
+    if (Dpkg::Build::Env::has('DEB_VENDOR')) {
+        $f = get_vendor_info(Dpkg::Build::Env::get('DEB_VENDOR'));
         return $f->{'Vendor'} if defined $f;
     }
     $f = get_vendor_info();
@@ -140,7 +143,7 @@ sub get_current_vendor() {
     return;
 }
 
-=item $object = Dpkg::Vendor::get_vendor_object($name)
+=item $object = get_vendor_object($name)
 
 Return the Dpkg::Vendor::* object of the corresponding vendor.
 If $name is omitted, return the object of the current vendor.
@@ -159,6 +162,7 @@ sub get_vendor_object {
 
     foreach my $name (@names) {
         eval qq{
+            pop \@INC if \$INC[-1] eq '.';
             require Dpkg::Vendor::$name;
             \$obj = Dpkg::Vendor::$name->new();
         };
@@ -176,7 +180,7 @@ sub get_vendor_object {
     }
 }
 
-=item Dpkg::Vendor::run_vendor_hook($hookid, @params)
+=item run_vendor_hook($hookid, @params)
 
 Run a hook implemented by the current vendor object.
 
@@ -199,6 +203,10 @@ New function: get_vendor_dir().
 
 Mark the module as public.
 
+=head1 SEE ALSO
+
+deb-origin(5).
+
 =cut
 
 1;
