@@ -2,7 +2,7 @@
 #
 # dpkg-scanpackages
 #
-# Copyright © 2006-2012 Guillem Jover <guillem@debian.org>
+# Copyright © 2006-2015 Guillem Jover <guillem@debian.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,19 +20,17 @@
 use warnings;
 use strict;
 
-use IO::Handle;
-use IO::File;
 use Getopt::Long qw(:config posix_default bundling no_ignorecase);
+use List::Util qw(none);
+use File::Find;
 
 use Dpkg ();
 use Dpkg::Gettext;
 use Dpkg::ErrorHandling;
-use Dpkg::Util qw(:list);
 use Dpkg::Control;
 use Dpkg::Version;
 use Dpkg::Checksums;
 use Dpkg::Compression::FileHandle;
-use Dpkg::IPC;
 
 textdomain('dpkg-dev');
 
@@ -43,11 +41,13 @@ my (@samemaint, @changedmaint);
 my @spuriousover;
 my %packages;
 my %overridden;
+my %hash;
 
 my %options = (help            => sub { usage(); exit 0; },
-	       version         => \&version,
+	       version         => sub { version(); exit 0; },
 	       type            => undef,
 	       arch            => undef,
+	       hash            => undef,
 	       multiversion    => 0,
 	       'extra-override'=> undef,
                medium          => undef,
@@ -58,23 +58,24 @@ my @options_spec = (
     'version',
     'type|t=s',
     'arch|a=s',
+    'hash|h=s',
     'multiversion|m!',
     'extra-override|e=s',
     'medium|M=s',
 );
 
 sub version {
-    printf _g("Debian %s version %s.\n"), $Dpkg::PROGNAME, $Dpkg::PROGVERSION;
-    exit;
+    printf g_("Debian %s version %s.\n"), $Dpkg::PROGNAME, $Dpkg::PROGVERSION;
 }
 
 sub usage {
-    printf _g(
+    printf g_(
 "Usage: %s [<option>...] <binary-path> [<override-file> [<path-prefix>]] > Packages
 
 Options:
   -t, --type <type>        scan for <type> packages (default is 'deb').
   -a, --arch <arch>        architecture to scan for.
+  -h, --hash <hash-list>   only generate hashes for the specified list.
   -m, --multiversion       allow multiple versions of a single package.
   -e, --extra-override <file>
                            use extra override file.
@@ -109,7 +110,7 @@ sub load_override
 		    my $debmaint = $$package{Maintainer};
 		    if (none { $debmaint eq $_ } split m{\s*//\s*}, $oldmaint) {
 			push(@changedmaint,
-			     sprintf(_g('  %s (package says %s, not %s)'),
+			     sprintf(g_('  %s (package says %s, not %s)'),
 			             $p, $$package{Maintainer}, $oldmaint));
 		    } else {
 			$$package{Maintainer} = $newmaint;
@@ -117,7 +118,7 @@ sub load_override
 		} elsif ($$package{Maintainer} eq $maintainer) {
 		    push(@samemaint, "  $p ($maintainer)");
 		} else {
-		    warning(_g('unconditional maintainer override for %s'), $p);
+		    warning(g_('unconditional maintainer override for %s'), $p);
 		    $$package{Maintainer} = $maintainer;
 		}
 	    }
@@ -152,100 +153,110 @@ sub load_override_extra
     close($comp_file);
 }
 
+sub process_deb {
+    my ($pathprefix, $fn) = @_;
+
+    my $fields = Dpkg::Control->new(type => CTRL_INDEX_PKG);
+
+    open my $output_fh, '-|', 'dpkg-deb', '-I', $fn, 'control'
+        or syserr(g_('cannot fork for %s'), 'dpkg-deb');
+    $fields->parse($output_fh, $fn)
+        or error(g_("couldn't parse control information from %s"), $fn);
+    close $output_fh;
+    if ($?) {
+        warning(g_("'dpkg-deb -I %s control' exited with %d, skipping package"),
+                $fn, $?);
+        return;
+    }
+
+    my $p = $fields->{'Package'};
+    error(g_('no Package field in control file of %s'), $fn)
+        if not defined $p;
+
+    if (defined($packages{$p}) and not $options{multiversion}) {
+        foreach my $pkg (@{$packages{$p}}) {
+            if (version_compare_relation($fields->{'Version'}, REL_GT,
+                                         $pkg->{'Version'}))
+            {
+                warning(g_('package %s (filename %s) is repeat but newer ' .
+                           'version; used that one and ignored data from %s!'),
+                        $p, $fn, $pkg->{Filename});
+                $packages{$p} = [];
+            } else {
+                warning(g_('package %s (filename %s) is repeat; ' .
+                           'ignored that one and using data from %s!'),
+                        $p, $fn, $pkg->{Filename});
+                return;
+            }
+        }
+    }
+
+    warning(g_('package %s (filename %s) has Filename field!'), $p, $fn)
+        if defined($fields->{'Filename'});
+    $fields->{'Filename'} = "$pathprefix$fn";
+
+    my $sums = Dpkg::Checksums->new();
+    $sums->add_from_file($fn);
+    foreach my $alg (checksums_get_list()) {
+        next if %hash and not $hash{$alg};
+
+        if ($alg eq 'md5') {
+            $fields->{'MD5sum'} = $sums->get_checksum($fn, $alg);
+        } else {
+            $fields->{$alg} = $sums->get_checksum($fn, $alg);
+        }
+    }
+    $fields->{'Size'} = $sums->get_size($fn);
+    $fields->{'X-Medium'} = $options{medium} if defined $options{medium};
+
+    push @{$packages{$p}}, $fields;
+}
+
 {
     local $SIG{__WARN__} = sub { usageerr($_[0]) };
     GetOptions(\%options, @options_spec);
 }
 
 if (not (@ARGV >= 1 and @ARGV <= 3)) {
-    usageerr(_g('one to three arguments expected'));
+    usageerr(g_('one to three arguments expected'));
 }
 
 my $type = $options{type} // 'deb';
 my $arch = $options{arch};
+%hash = map { $_ => 1 } split /,/, $options{hash} // '';
 
-my @find_args;
-if ($options{arch}) {
-     @find_args = ('(', '-name', "*_all.$type", '-o',
-			'-name', "*_${arch}.$type", ')');
-}
-else {
-     @find_args = ('-name', "*.$type");
+foreach my $alg (keys %hash) {
+    if (not checksums_is_supported($alg)) {
+        usageerr(g_('unsupported checksum \'%s\''), $alg);
+    }
 }
 
-my ($binarydir, $override, $pathprefix) = @ARGV;
+my ($binarypath, $override, $pathprefix) = @ARGV;
 
-if (not -d $binarydir) {
-    error(_g('binary dir %s not found'), $binarydir);
+if (not -e $binarypath) {
+    error(g_('binary path %s not found'), $binarypath);
 }
 if (defined $override and not -e $override) {
-    error(_g('override file %s not found'), $override);
+    error(g_('override file %s not found'), $override);
 }
 
 $pathprefix //= '';
 
-my $find_h = IO::Handle->new();
-open($find_h, '-|', 'find', '-L', "$binarydir/", @find_args, '-print')
-     or syserr(_g("couldn't open %s for reading"), $binarydir);
-FILE:
-    while (<$find_h>) {
-	chomp;
-	my $fn = $_;
-	my $output;
-	my $pid = spawn(exec => [ 'dpkg-deb', '-I', $fn, 'control' ],
-	                to_pipe => \$output);
-	my $fields = Dpkg::Control->new(type => CTRL_INDEX_PKG);
-	$fields->parse($output, $fn)
-	    or error(_g("couldn't parse control information from %s"), $fn);
-	wait_child($pid, nocheck => 1);
-	if ($?) {
-	    warning(_g("\`dpkg-deb -I %s control' exited with %d, skipping package"),
-	            $fn, $?);
-	    next;
-	}
+my $find_filter;
+if ($options{arch}) {
+    $find_filter = qr/_(?:all|${arch})\.$type$/;
+} else {
+    $find_filter = qr/\.$type$/;
+}
+my @archives;
+my $scan_archives = sub {
+    push @archives, $File::Find::name if m/$find_filter/;
+};
 
-	defined($fields->{'Package'})
-	    or error(_g('no Package field in control file of %s'), $fn);
-	my $p = $fields->{'Package'};
-
-	if (defined($packages{$p}) and not $options{multiversion}) {
-	    foreach (@{$packages{$p}}) {
-		if (version_compare_relation($fields->{'Version'}, REL_GT,
-					     $_->{'Version'}))
-                {
-		    warning(_g('package %s (filename %s) is repeat but newer version;'),
-		            $p, $fn);
-		    warning(_g('used that one and ignored data from %s!'),
-		            $_->{Filename});
-		    $packages{$p} = [];
-		} else {
-		    warning(_g('package %s (filename %s) is repeat;'), $p, $fn);
-		    warning(_g('ignored that one and using data from %s!'),
-		            $_->{Filename});
-		    next FILE;
-		}
-	    }
-	}
-	warning(_g('package %s (filename %s) has Filename field!'), $p, $fn)
-	    if defined($fields->{'Filename'});
-
-	$fields->{'Filename'} = "$pathprefix$fn";
-
-        my $sums = Dpkg::Checksums->new();
-	$sums->add_from_file($fn);
-        foreach my $alg (checksums_get_list()) {
-            if ($alg eq 'md5') {
-	        $fields->{'MD5sum'} = $sums->get_checksum($fn, $alg);
-            } else {
-                $fields->{$alg} = $sums->get_checksum($fn, $alg);
-            }
-        }
-	$fields->{'Size'} = $sums->get_size($fn);
-        $fields->{'X-Medium'} = $options{medium} if defined $options{medium};
-
-	push @{$packages{$p}}, $fields;
-    }
-close($find_h);
+find({ follow => 1, follow_skip => 2, wanted => $scan_archives}, $binarypath);
+foreach my $fn (@archives) {
+    process_deb($pathprefix, $fn);
+}
 
 load_override($override) if defined $override;
 load_override_extra($options{'extra-override'}) if defined $options{'extra-override'};
@@ -257,28 +268,28 @@ for my $p (sort keys %packages) {
     if (defined($override) and not defined($overridden{$p})) {
         push @missingover, $p;
     }
-    for my $package (@{$packages{$p}}) {
-         print("$package\n") or syserr(_g('failed when writing stdout'));
+    for my $package (sort { $a->{Version} cmp $b->{Version} } @{$packages{$p}}) {
+         print("$package\n") or syserr(g_('failed when writing stdout'));
          $records_written++;
     }
 }
-close(STDOUT) or syserr(_g("couldn't close stdout"));
+close(STDOUT) or syserr(g_("couldn't close stdout"));
 
 if (@changedmaint) {
-    warning(_g('Packages in override file with incorrect old maintainer value:'));
+    warning(g_('Packages in override file with incorrect old maintainer value:'));
     warning($_) foreach (@changedmaint);
 }
 if (@samemaint) {
-    warning(_g('Packages specifying same maintainer as override file:'));
+    warning(g_('Packages specifying same maintainer as override file:'));
     warning($_) foreach (@samemaint);
 }
 if (@missingover) {
-    warning(_g('Packages in archive but missing from override file:'));
+    warning(g_('Packages in archive but missing from override file:'));
     warning('  %s', join(' ', @missingover));
 }
 if (@spuriousover) {
-    warning(_g('Packages in override file but not in archive:'));
+    warning(g_('Packages in override file but not in archive:'));
     warning('  %s', join(' ', @spuriousover));
 }
 
-info(_g('Wrote %s entries to output Packages file.'), $records_written);
+info(g_('Wrote %s entries to output Packages file.'), $records_written);
