@@ -256,15 +256,15 @@ trigproc_new_cyclenode(struct pkginfo *processing_now)
 	struct trigcyclenode *tcn;
 	struct trigcycleperpkg *tcpp;
 	struct pkginfo *pkg;
-	struct pkgiterator *it;
+	struct pkgiterator *iter;
 
 	tcn = nfmalloc(sizeof(*tcn));
 	tcn->pkgs = NULL;
 	tcn->next = NULL;
 	tcn->then_processed = processing_now;
 
-	it = pkg_db_iter_new();
-	while ((pkg = pkg_db_iter_next_pkg(it))) {
+	iter = pkg_db_iter_new();
+	while ((pkg = pkg_db_iter_next_pkg(iter))) {
 		if (!pkg->trigpend_head)
 			continue;
 		tcpp = nfmalloc(sizeof(*tcpp));
@@ -273,7 +273,7 @@ trigproc_new_cyclenode(struct pkginfo *processing_now)
 		tcpp->next = tcn->pkgs;
 		tcn->pkgs = tcpp;
 	}
-	pkg_db_iter_free(it);
+	pkg_db_iter_free(iter);
 
 	return tcn;
 }
@@ -513,11 +513,11 @@ transitional_interest_callback(const char *trig, struct pkginfo *pkg,
 static void
 trig_transitional_activate(enum modstatdb_rw cstatus)
 {
-	struct pkgiterator *it;
+	struct pkgiterator *iter;
 	struct pkginfo *pkg;
 
-	it = pkg_db_iter_new();
-	while ((pkg = pkg_db_iter_next_pkg(it))) {
+	iter = pkg_db_iter_new();
+	while ((pkg = pkg_db_iter_next_pkg(iter))) {
 		if (pkg->status <= PKG_STAT_HALFINSTALLED)
 			continue;
 		debug(dbg_triggersdetail, "trig_transitional_activate %s %s",
@@ -544,7 +544,7 @@ trig_transitional_activate(enum modstatdb_rw cstatus)
 		else
 			pkg_set_status(pkg, PKG_STAT_INSTALLED);
 	}
-	pkg_db_iter_free(it);
+	pkg_db_iter_free(iter);
 
 	if (cstatus >= msdbrw_write) {
 		modstatdb_checkpoint();
